import imp
import os
import pickle
import string
import sys
import types


# Cache the .py file module description.
for d in imp.get_suffixes():
    if (d[0] == '.py'):
       PYFILE_MODULE_DESCRIPTION = d
       break
    # end if
# end for
del d


class SimpleRpcRequest:

    """A data container for a very simple remote procedure call.

    For details on the types of procedure calls supported, see the
    __init__ method.

    Public methods:
    invoke
    """

    def __init__(self, f, a): 
	"""Take a function object and a tuple of actuals (arguments).

	The function to call must be defined at top-level in a module.
	That module must reside in a single .py file, and the path to
	that file must be the same on both the local and remote
	systems.  This implementation also does not support using
	keyword arguments in the actuals to the function.

	Stay away from using complicated objects in the actuals;
	objects with external representations (e.g., Tkinter widgets,
	open file objects) won't work here for obvious reasons.

	Note this does not implement any particular transport
	mechanism.  You need something else to actually send this
	information over the network and start up Python with the
	appropriate sys.path, pwd, etc.

	"""

	if (type(f) != types.FunctionType):
	    raise ValueError, 'f is not a function'
	elif (type(a) != types.TupleType):
	    raise ValueError, 'a is not a tuple'
	# end if

	self._modulePath = f.func_code.co_filename
	self._functionName = f.func_name
	self._actuals = a

	# Only supported module type is '.py'.
	self._moduleDescription = PYFILE_MODULE_DESCRIPTION

	# Compute the module's name from the path.
	if (self._modulePath[-3:] != ".py"):
	    raise ValueError, ("f resides in a module with "+
		"an unsupported file type; must be in a .py file")
	# end if

	self._moduleName = self._modulePath[
	    string.rfind( self._modulePath, os.sep)+1 : -3]

	# This test fails if the function is defined in __main__, but
	# it shouldn't.
	#try:
	#    x = eval(self._moduleName + '.' + self._functionName)
	#except:
	#    raise ValueError, ("function " + self._functionName + 
	#	"does not appeared to be defined at top-level in "
	#	"module " + self._moduleName)
	# end try
    # end def __init__

    def invoke(self, preArgs=(), postArgs=(), **kw):
	"""Call the procedure and return its result.

	Meant to be used on the target system.  You may provide extra
	arguments before or after the actuals given in the
	constructor; you may also add keyword arguments.

	"""

	# Check args
	if (type(preArgs) != types.TupleType):
	    raise ValueError, 'preArgs is not a tuple'
	elif (type(postArgs) != types.TupleType):
	    raise ValueError, 'postArgs is not a tuple'
	# end if

	modfile = open(self._modulePath, self._moduleDescription[1])
	try:
	    modobj = imp.load_module(self._moduleName, modfile,
				     self._modulePath,
				     self._moduleDescription)
	finally:
	    modfile.close()
	# end try

	func = modobj.__dict__[self._functionName]
	return apply(func, preArgs + self._actuals + postArgs, kw)
    # end def invoke

# end class SimpleRpcRequest


######################################################################
# Testing.

#import myMath

def _testOut(filename):
    import myMath
    file = open(filename, "w")
    pickle.dump(SimpleRpcRequest(myMath.sqrt, (42.0,)), 
		file)
    file.close()
# end def _testOut


def _testIn(filename):
    file = open(filename, "r")
    req = pickle.load(file)
    file.close()

    return req.invoke()
# end def _testIn


#_testOut("sqrt42.rpc")
#_testIn("sqrt42.rpc")
