# I wrote this because socket.socket.makefile() wasn't working the way
# I wanted it to.  I also wanted to debug my socket connections.


import socket

class InheritableSocket:
    """A socket-like object from which other sockets can be inherited.

    This works around a problem with subclassing socket.socket:
    accept() only produces other socket objects.  How rude!
    There is one drawback:  it doesn't yet support makefile().

    To subclass properly, pass a socketClass keyword with the value of
    the class you want to use for the new socket-like object.  That
    class's constructor will be called with a realSocket keyword.  You
    can then pass that keyword up to InheritableSocket.__init__().

    """

    def __init__(self, *args, **kw):
	"""InheritableSocket constructor.

	keyword 'realSocket' - a socket.socket "instance" to use

	If that keyword is not given, pass all arguments to
	socket.socket() to create a new instance instead.

	"""

        if 'realSocket' in kw.keys():
	    self._s = kw['realSocket']
	else:
	    self._s = apply(socket.socket, args, kw)
	# end if

	x = self._s
	self.__delegationDict = {
	    'bind':x.bind, 
	    'close':x.close,
	    'connect':x.connect,
	    'fileno':x.fileno,
	    'listen':x.listen,
	    'send':x.send,
	    'recv':x.recv
	}
    # end def __init__

    def __getattr__(self, name):
	"""Fast delegated inheritance."""

	try:
	    return self.__dict__[name]
	except KeyError:
	    try:
	        return self.__delegationDict[name]
	    except KeyError:
	        raise AttributeError, name
	    # end try	
	# end try
    # end def __getattr__

    def accept(self, socketClass=None):
	if (socketClass is None):
	    socketClass = self.__class__
	# end if

	(conn, addr) = self._s.accept()
	fakeConn = socketClass(realSocket=conn)
	return (fakeConn, addr)
    # end def accept

#    def bind(self, *args):
#	return apply(self._s.bind, args)
#    # end def bind
#
#    def close(self, *args):
#	return apply(self._s.close, args)
#    # end def close
#
#    def connect(self, *args):
#	return apply(self._s.connect, args)
#    # end def connect
#
#    def listen(self, *args):
#	return apply(self._s.listen, args)
#    # end def connect

# end class InheritableSocket


class DebugSocket(InheritableSocket):

    def accept(self, socketClass=None):
	(conn, addr) = InheritableSocket.accept(self, socketClass=socketClass)
	print "DebugSocket:", conn, "connected to", addr
	return (conn, addr)
    # end def accept

    def connect(self, *args):
	result = apply(self._s.connect, args)
	print "DebugSocket:", self, "connected to", args
	return result
    # end def connect

    def send(self, *args):
	result = apply(self._s.send, args)
	print "DebugSocket:", self, "->", repr(args[0])
	return result
    # end def send

    def recv(self, *args):
	result = apply(self._s.recv, args)
	print "DebugSocket:", self, "<-", repr(result)
	return result
    # end def recv

# end class DebugSocket


# Defines just enough file-like methods to make the pickle module happy.

#class FileSocket(DebugSocket):
class FileSocket(InheritableSocket):

    def read(self, max=-1):
        result = ""
	numRead = 0
	while (max < 0 or numRead < max):
	    if (max < 0):
	        s = self.recv(65536)
	    else:
	        s = self.recv(max - numRead)
	    # end if
	    if (s == ''):
	        # EOF reached.
		return result
	    # end if
	    numRead = numRead + len(s)
	    result = result + s
	# end while 
	return result
    # end def read

    def readline(self, max=-1):
        result = ""
	numRead = 0
	while ((max < 0 or numRead < max) and
	       (not result or result[-1] != '\n')):
	    c = self.recv(1)
	    if (c == ''):
	        # EOF reached.
		return result
	    # end if
	    numRead = numRead + 1
	    result = result + c
	# end while
	return result
    # end def readline

    def write(self, s):
	self.send(s)
    # end def write

# end class FileSocket
